package dk.nsp.xdscleanup.util.documentutil;

import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLProvideAndRegisterDocumentSetRequest30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.ProvideAndRegisterDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.*;
import org.openehealth.ipf.commons.ihe.xds.core.requests.ProvideAndRegisterDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.ProvideAndRegisterDocumentSetTransformer;

import javax.activation.DataHandler;
import java.util.List;

public class ProvideAndRegisterDocumentSetUtilities extends AbstractXdsUtilities {

	
	public ProvideAndRegisterDocumentSetRequestType buildProvideAndRegisterDocumentSetRequest(DocumentEntry documentEntry, String documentPayload, DocumentMetadata documentMetadata, String externalIdForDocumentToReplace) {
		return buildProvideAndRegisterDocumentSetRequest(documentEntry, documentPayload, externalIdForDocumentToReplace, documentMetadata.getContentTypeCode());
	}
	
	public ProvideAndRegisterDocumentSetRequestType buildProvideAndRegisterDocumentSetRequest(DocumentEntry documentEntry, String documentPayload, String externalIdForDocumentToReplace, Code contentTypeCode) {

		ProvideAndRegisterDocumentSet provideAndRegisterDocumentSet = new ProvideAndRegisterDocumentSet();

		Document document = new Document(documentEntry, new DataHandler(documentPayload.getBytes(), documentEntry.getMimeType()));
		provideAndRegisterDocumentSet.getDocuments().add(document);

		// Opret SubmissionSet for dokumentet
		SubmissionSet submissionSet = createSubmissionSet(documentEntry.getPatientId(), contentTypeCode, documentEntry.getCreationTime());
		provideAndRegisterDocumentSet.setSubmissionSet(submissionSet);

		// Opret association mellem SubmissionSet og DocumentEntry
		Association association = createHasMemberAssociation(submissionSet, documentEntry);
		provideAndRegisterDocumentSet.getAssociations().add(association);
		
		// Hvis det er en opdatering af dokumentet, oprettes association mellem det gamle og det nye dokument vha. entryuuid'erne
		if (externalIdForDocumentToReplace != null) {
			Association replacementAssociation = new Association(AssociationType.REPLACE, generateUUID(), documentEntry.getEntryUuid(), externalIdForDocumentToReplace);
			provideAndRegisterDocumentSet.getAssociations().add(replacementAssociation);
		}

		// Transformer request - dette laver core model om til RIM format
		ProvideAndRegisterDocumentSetTransformer registerDocumentSetTransformer = new ProvideAndRegisterDocumentSetTransformer(ebXMLFactory);
		EbXMLProvideAndRegisterDocumentSetRequest30 ebxmlRequest = (EbXMLProvideAndRegisterDocumentSetRequest30) registerDocumentSetTransformer.toEbXML(provideAndRegisterDocumentSet);
		ProvideAndRegisterDocumentSetRequestType provideAndRegisterDocumentSetRequestType = ebxmlRequest.getInternal();
	
		return provideAndRegisterDocumentSetRequestType;
	}

	public DocumentEntry createDocumentEntry(DocumentMetadata documentMetadata, String documentEntryUuid, String uniqueId, String logicalId) {
		String reportTime = (documentMetadata.getReportTime() == null ) ? null : dateTimeFormat.format(documentMetadata.getReportTime()); 
		String serviceStartTime = (documentMetadata.getServiceStartTime() == null ) ? null : dateTimeFormat.format(documentMetadata.getServiceStartTime());
		String serviceStopTime = (documentMetadata.getServiceStopTime() == null ) ? null : dateTimeFormat.format(documentMetadata.getServiceStopTime());
		
		return createDocumentEntry(documentMetadata.getPatientId(), documentMetadata.getOrganisation(), reportTime, serviceStartTime, serviceStopTime, 
				documentMetadata.getClassCode(), documentMetadata.getConfidentialityCode(), documentMetadata.getEventCodes(), documentMetadata.getFormatCode(), documentMetadata.getHealthcareFacilityTypeCode(), 
				documentMetadata.getLanguageCode(), documentMetadata.getMimeType(), documentMetadata.getTitle(), documentMetadata.getTypeCode(), documentMetadata.getPracticeSettingCode(), documentMetadata.getSourcePatientInfo(),   
				documentEntryUuid, uniqueId, logicalId);
	}
	
	public DocumentEntry createDocumentEntry(Code patientId, Code organisation, String creationTime, String serviceStartTime, String serviceStopTime, Code classCode, Code confidentialityCode, List<Code> eventCodes, Code formatCode, Code healthcareFacilityTypeCode, 
			 String languageCode, String mimeType, String title, Code typeCode, Code practiceSettingCode, PatientInfo sourcePatientInfo, String documentEntryUuid, String uniqueId, String logicalId) {
		
		// Opret DocumentEntry for CDA dokumentet
		DocumentEntry documentEntry = new DocumentEntry();
		documentEntry.setEntryUuid(documentEntryUuid);
		documentEntry.setUniqueId(uniqueId);
		documentEntry.setLogicalUuid(logicalId);

		// Patient Identification
		Identifiable patientIdentifiable = null;
		if (patientId != null) {
			AssigningAuthority patientIdAssigningAuthority = new AssigningAuthority(patientId.getSchemeName());
			patientIdentifiable = new Identifiable(patientId.getCode(), patientIdAssigningAuthority);
		}
		documentEntry.setPatientId(patientIdentifiable);
		documentEntry.setSourcePatientId(patientIdentifiable);
		
        if (sourcePatientInfo != null) {
        	documentEntry.setSourcePatientInfo(sourcePatientInfo);	
        }
 
		// Opret author (Organisation)
		Author author = new Author();
		if (organisation != null && organisation.getCode() != null) {
			AssigningAuthority organisationAssigningAuthority = new AssigningAuthority(organisation.getSchemeName());
			String orgDisplayname = (organisation.getDisplayName() != null ? organisation.getDisplayName().getValue() : "");
			Organization authorOrganisation = new Organization(orgDisplayname, organisation.getCode(), organisationAssigningAuthority);
			author.getAuthorInstitution().add(authorOrganisation);
		}
		documentEntry.setAuthor(author);

		// Availability Status (enumeration: APPROVED, SUBMITTED, DEPRECATED)
		documentEntry.setAvailabilityStatus(AvailabilityStatus.APPROVED);

		// Datoer - skal angives i UTC tid
		if (creationTime != null) {
			documentEntry.setCreationTime(creationTime);
		}
		if (serviceStartTime != null) {
			documentEntry.setServiceStartTime(serviceStartTime);
		}
		if (serviceStopTime != null) {
			documentEntry.setServiceStopTime(serviceStopTime);
		}

		if (classCode != null) {
			documentEntry.setClassCode(classCode);
		}
		if (confidentialityCode != null) {
			documentEntry.getConfidentialityCodes().add(confidentialityCode);
		}

		List<Code> eventCodesEntry = documentEntry.getEventCodeList();
		if (eventCodes != null) {
			for (Code eventCode : eventCodes) {
				eventCodesEntry.add(eventCode);
			}
		}
		if (formatCode != null) {
			documentEntry.setFormatCode(formatCode);
		}
		if (healthcareFacilityTypeCode != null) {
			documentEntry.setHealthcareFacilityTypeCode(healthcareFacilityTypeCode);
		}
		if (languageCode != null) {
			documentEntry.setLanguageCode(languageCode);
		}
		if (mimeType != null) {
			documentEntry.setMimeType(mimeType);
		}
		documentEntry.setType(DocumentEntryType.STABLE);
		if (title != null) {
			documentEntry.setTitle(new LocalizedString(title));
		}
		if (typeCode != null) {
			documentEntry.setTypeCode(typeCode);
		}
		if (practiceSettingCode != null) {
			documentEntry.setPracticeSettingCode(practiceSettingCode);
		}
		return documentEntry;
	}

	public SubmissionSet createSubmissionSet(Identifiable patientIdentifiable, Code contentTypeCode, DateTime submissionTime) {
		String submissionSetUuid = generateId();
		String submissionTimeString = (submissionTime == null ) ? null : submissionTime.toString("yyyyMMddHHmmss");
		return createSubmissionSet(patientIdentifiable, contentTypeCode, submissionTimeString, submissionSetUuid, submissionSetUuid, submissionSetUuid);
	}

	public SubmissionSet createSubmissionSet(Identifiable patientIdentifiable, Code contentTypeCode, String submissionTime, String entryUuid, String uniqueId, String sourceId) {
		SubmissionSet submissionSet = new SubmissionSet();
		submissionSet.setUniqueId(uniqueId);
		submissionSet.setSourceId(sourceId);
		submissionSet.setEntryUuid(entryUuid);
		submissionSet.setPatientId(patientIdentifiable);
		submissionSet.setContentTypeCode(contentTypeCode);
		submissionSet.setAvailabilityStatus(AvailabilityStatus.APPROVED);

		if (submissionTime != null) {
			submissionSet.setSubmissionTime(submissionTime);
		}
		return submissionSet;
	}

}
